package de.intevation.mxd.reader;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.IFillSymbol;
import com.esri.arcgis.display.SimpleFillSymbol;
import com.esri.arcgis.display.esriSimpleFillStyle;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import org.w3c.dom.Element;
import java.io.IOException;

/**
 * Reads simple marker symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class SimpleFillSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(SimpleFillSymbolReader.class);

    /**
     * Private member.
     */
    private SimpleFillSymbol symbol;

    public SimpleFillSymbolReader(ISymbol symbol) throws Exception{
        logger.debug("contructor(ISymbol)");
        if(symbol instanceof SimpleFillSymbol) {
            this.symbol = (SimpleFillSymbol)symbol;
        }
        else {
            throw new Exception("Not a SimpleFillSymbol!");
        }
    }

    public SimpleFillSymbolReader(IFillSymbol symbol) throws Exception{
        logger.debug("contructor(IFillSymbol)");
        if(symbol instanceof SimpleFillSymbol) {
            this.symbol = (SimpleFillSymbol)symbol;
        }
        else {
            throw new Exception("Not a SimpleFillSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read() {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(parent);

        try {
            symbolElement.setAttribute("name", symbol.getNameString());
        }
        catch(IOException ioe) {
            logger.warn("Could not read name. Setting name to \"default\"");
            symbolElement.setAttribute("name", "default");
        }
        symbolElement.setAttribute("style", "fill");
        int style;
        try {
            style = symbol.getStyle();
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read fill style." +
                " Setting fill style to \"empty\"");
            style = -1;
        }

        switch(style) {
            case esriSimpleFillStyle.esriSFSCross:
                symbolElement.setAttribute("fillstyle", "cross"); break;
            case esriSimpleFillStyle.esriSFSSolid:
                symbolElement.setAttribute("fillstyle", "solid"); break;
            case esriSimpleFillStyle.esriSFSVertical:
                symbolElement.setAttribute("fillstyle", "vertical"); break;
            case esriSimpleFillStyle.esriSFSHorizontal:
                symbolElement.setAttribute("fillstyle", "horizontal"); break;
            case esriSimpleFillStyle.esriSFSForwardDiagonal:
                symbolElement.setAttribute("fillstyle", "fwdiagonal"); break;
            case esriSimpleFillStyle.esriSFSBackwardDiagonal:
                symbolElement.setAttribute("fillstyle", "bwdiagonal"); break;
            case esriSimpleFillStyle.esriSFSDiagonalCross:
                symbolElement.setAttribute("fillstyle", "diagonalcross");
                break;
            default: symbolElement.setAttribute("fillstyle", "empty");
        }

        try {
            if(symbol.getColor() instanceof IRgbColor) {
                IRgbColor color = (IRgbColor)symbol.getColor();
                Color c = new Color (
                    color.getRed(),
                    color.getGreen(),
                    color.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
                symbolElement.setAttribute("transparency",
                    String.valueOf(color.getTransparency()));
            }
            else {
                RgbColor col = new RgbColor();
                col.setRGB(symbol.getColor().getRGB());
                Color c = new Color (
                    col.getRed(),
                    col.getGreen(),
                    col.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
                symbolElement.setAttribute("transparency",
                    String.valueOf(col.getTransparency()));
            }
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read color." +
                " Setting color to black with no transparency");
            Color black = new Color(0, 0, 0);
            symbolElement.setAttribute("color", String.valueOf(black.getRGB()));
            symbolElement.setAttribute("transparency", "-1");
        }

        try {
            ILineSymbol ls = symbol.getOutline();
            LineSymbolReader lsr = new LineSymbolReader();
            if(lsr.canRead(ls)) {
                lsr.setSymbol(ls);
                lsr.setUtil(util);
                lsr.setParent(symbolElement);
                lsr.read();
            }
            else {
                logger.debug("The type of " + ls.getClass().toString() +
                             " is not implemented!");
            }
        }
        catch(Exception e) {
            logger.warn("Could not read line symbol. No fallback defined.");
        }

        return symbolElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
